use std::sync::Arc;
use axum::{
    extract::{State},
};
use axum::Json;
use serde::{Deserialize,Serialize};
use crate::*;
use crate::PowerDnsOidcTsigkeyError;

pub async fn list_keys(
    State(state):  State<Arc<AppState>>,
) -> Result<Json<Vec<TsigKey>>, PowerDnsOidcTsigkeyError> {
    let req = state.http_client.get::<String>((config_cell.get().unwrap().powerdns.url.to_string() + "/servers/localhost/tsigkeys").into())
        .header("X-API-Key", config_cell.get().unwrap().powerdns.api_token.clone());
    let response = req
        .send()
        .await?;
    match response.status().is_success() {
        true => {
            let powerdns_tsig_keys: Vec<PowerDnsTsigKey> = response.json().await?;
            Ok(Json(powerdns_tsig_keys.into_iter().map(|key| key.into()).collect()))
        },
        false => Err(PowerDnsOidcTsigkeyError::from_response(response).await)
    }
}
//pub async fn list_key(Path(key_id): Path<String>) -> Json<TsigKey> {}
//pub async fn create_key() -> Json<TsigKey> {}
//pub async fn delete_key(Path(key_id): Path<String>) {}


#[derive(Serialize, Debug)]
pub struct TsigKeyList {
    pub keys: Vec<TsigKey>,
}

#[derive(Serialize, Debug)]
pub struct TsigKey {
    pub name: String,
    pub algorithm: String,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub key: Option<String>,
}

#[derive(Deserialize, Serialize, Debug)]
struct PowerDnsTsigKey {
    pub name: String,
    pub id: String,
    pub algorithm: String,
    pub key: String,
    #[serde(rename = "type")]
    pub object_type: String,
}

impl From<PowerDnsTsigKey> for TsigKey {
    fn from(tsigkey: PowerDnsTsigKey) -> Self {
        Self {
            name: tsigkey.name,
            algorithm: tsigkey.algorithm,
            key: Some(tsigkey.key).filter(|s| !s.is_empty())
        }
    }
}

